@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.2, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.2
+# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.4
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2024 the gcovr authors
+# Copyright (c) 2013-2025 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -21,13 +21,14 @@
 
 import logging
 import re
-from typing import List
+from typing import Optional
 
-from .coverage import (
+from .data_model.coverage import (
     DecisionCoverageUncheckable,
     DecisionCoverageConditional,
     DecisionCoverageSwitch,
     FileCoverage,
+    LineCoverage,
 )
 
 LOGGER = logging.getLogger("gcovr")
@@ -72,7 +73,7 @@ def _prepare_decision_string(code: str) -> str:
     return " " + code.lstrip().strip()
 
 
-def _get_delta_braces(code):
+def _get_delta_braces(code: str) -> int:
     prepared_string = _prepare_decision_string(code)
     return prepared_string.count("(") - prepared_string.count(")")
 
@@ -160,9 +161,15 @@ class DecisionParser:
             The encoding of the source files
     """
 
-    def __init__(self, coverage: FileCoverage, lines: List[str]):
-        self.coverage: FileCoverage = coverage
-        self.lines: List[str] = lines
+    def __init__(self, filecov: FileCoverage, lines: list[str]) -> None:
+        # If there are several line coverage definitions for the same line we ignore all of them
+        self.linecov_by_line: dict[int, Optional[LineCoverage]] = {}
+        for linecov_collection in filecov.lines():
+            if len(linecov_collection) == 1:
+                self.linecov_by_line[linecov_collection.lineno] = list(
+                    linecov_collection.linecov()
+                )[0]
+        self.lines = lines
 
         # status variables for decision analysis
         self.decision_analysis_active: bool = (
@@ -171,24 +178,26 @@ class DecisionParser:
         self.last_decision_line: int = 0
         self.decision_analysis_open_brackets: int = 0
 
-    def parse_all_lines(self):
+    def parse_all_lines(self) -> None:
+        """Parse the decisions of the file."""
         LOGGER.debug("Starting the decision analysis")
 
         # start to iterate through the lines
         for lineno, code in enumerate(self.lines, 1):
-            self.parse_one_line(lineno, code)
+            self._parse_one_line(lineno, code)
 
         LOGGER.debug("Decision Analysis finished!")
 
-    def parse_one_line(self, lineno: int, code: str) -> None:
-        line_coverage = self.coverage.lines.get(lineno)
+    def _parse_one_line(self, lineno: int, code: str) -> None:
+        """Parse a single line"""
+        linecov = self.linecov_by_line.get(lineno)
 
-        if line_coverage is None and not _is_a_switch(code):
+        if linecov is None and not _is_a_switch(code):
             return
 
         # check, if a analysis for a classic if-/else if-branch is active
         if self.decision_analysis_active:
-            self.continue_multiline_decision_analysis(lineno, code)
+            self._continue_multiline_decision_analysis(lineno, code)
 
         # if no decision analysis is active, check the active line of code for a branch_statement or a loop
         if self.decision_analysis_active:
@@ -198,47 +207,52 @@ class DecisionParser:
             return
 
         # check if a branch exists (prevent misdetection caused by inaccurate parsing)
-        if line_coverage and len(line_coverage.branches.items()) > 0:
+        if linecov and linecov.has_reportable_branches:
+            branchcov_list = list(linecov.branches())
             if (
                 _is_a_loop(code)
                 or _is_a_oneline_branch(code)
-                or (
-                    _is_a_closed_branch(code)
-                    and (len(line_coverage.branches.items()) == 2)
-                )
+                or (_is_a_closed_branch(code) and (len(branchcov_list) == 2))
             ):
-                if len(line_coverage.branches.items()) == 2:
-                    keys = sorted(line_coverage.branches)
+                if len(branchcov_list) == 2:
                     # if it's a compact decision, we can only use the fallback to analyze
                     # simple decisions via branch calls
-                    line_coverage.decision = DecisionCoverageConditional(
-                        line_coverage.branches[keys[0]].count,
-                        line_coverage.branches[keys[1]].count,
+                    linecov.decision = DecisionCoverageConditional(
+                        linecov,
+                        linecov.data_sources,
+                        count_true=branchcov_list[0].count,
+                        count_false=branchcov_list[1].count,
                     )
                 else:
                     # it's a complex decision with more than 2 branches. No accurate detection possible
                     # Set the decision to uncheckable
-                    line_coverage.decision = DecisionCoverageUncheckable()
+                    linecov.decision = DecisionCoverageUncheckable(
+                        linecov,
+                        linecov.data_sources,
+                    )
                     LOGGER.debug(f"Uncheckable decision at line {lineno}")
             else:
-                self.start_multiline_decision_analysis(lineno, code)
+                self._start_multiline_decision_analysis(lineno, code)
 
         # check if it's a case statement (measured at every line of a case, so a branch definition isn't given)
         elif _is_a_switch(code):
             # Get the coverage of the next line before a break
             max_lineno = lineno + 1
-            if self.coverage.lines:
-                max_lineno = max(max_lineno, *self.coverage.lines.keys())
+            if self.linecov_by_line:
+                max_lineno = max(max_lineno, *self.linecov_by_line.keys())
 
             for next_lineno in range(lineno, max_lineno):
-                line_coverage = self.coverage.lines.get(next_lineno)
-                if line_coverage is not None:
-                    line_coverage.decision = DecisionCoverageSwitch(line_coverage.count)
+                linecov = self.linecov_by_line.get(next_lineno)
+                if linecov is not None:
+                    linecov.decision = DecisionCoverageSwitch(
+                        linecov, linecov.data_sources, count=linecov.count
+                    )
                     break
                 if " break ;" in _prepare_decision_string(code):
                     break
 
-    def start_multiline_decision_analysis(self, lineno: int, code: str) -> None:
+    def _start_multiline_decision_analysis(self, lineno: int, code: str) -> None:
+        """Handler for start of a decision written over several lines."""
         # normal (non-compact) branch, analyze execution of following lines
         self.decision_analysis_active = True
         self.last_decision_line = lineno
@@ -246,23 +260,32 @@ class DecisionParser:
         # count brackets to make sure we're outside of the decision expression
         self.decision_analysis_open_brackets += _get_delta_braces(code)
 
-    def continue_multiline_decision_analysis(self, lineno: int, code: str) -> None:
-        line_coverage = self.coverage.lines.get(lineno)
-        exec_count = 0 if line_coverage is None else line_coverage.count
-        last_decision_line_cov = self.coverage.lines.get(self.last_decision_line)
+    def _continue_multiline_decision_analysis(self, lineno: int, code: str) -> None:
+        """Handler for a decision which is continued on the current line."""
+        linecov = self.linecov_by_line.get(lineno)
+        exec_count = 0 if linecov is None else linecov.count
+        last_decision_linecov = self.linecov_by_line.get(self.last_decision_line)
+        if last_decision_linecov is None:
+            raise AssertionError(
+                "Sanity check failed, last decision must be present for multi line analysis."
+            )
 
         # check, if the branch statement was finished in the last line
         if self.decision_analysis_open_brackets == 0:
             # set execution counts for the decision. true is the exec_count.
             # false is the delta between executed blocks and executions of the decision statement.
-            delta_count = last_decision_line_cov.count - exec_count
+            delta_count = last_decision_linecov.count - exec_count
             if delta_count >= 0:
-                last_decision_line_cov.decision = DecisionCoverageConditional(
-                    exec_count,
-                    delta_count,
+                last_decision_linecov.decision = DecisionCoverageConditional(
+                    linecov,
+                    "unknown" if linecov is None else linecov.data_sources,
+                    count_true=exec_count,
+                    count_false=delta_count,
                 )
             else:
-                last_decision_line_cov.decision = DecisionCoverageUncheckable()
+                last_decision_linecov.decision = DecisionCoverageUncheckable(
+                    linecov, "unknown" if linecov is None else linecov.data_sources
+                )
                 LOGGER.debug(
                     f"Uncheckable decision at line {lineno}. (Delta = {delta_count})"
                 )
