@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 8.2, a parsing and reporting tool for gcov.
-# https://gcovr.com/en/8.2
+# This file is part of gcovr 8.4, a parsing and reporting tool for gcov.
+# https://gcovr.com/en/8.4
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2024 the gcovr authors
+# Copyright (c) 2013-2025 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -18,11 +18,15 @@
 # ****************************************************************************
 
 import logging
-from typing import Callable, List, Optional, Tuple
+from typing import Callable, Optional
 
-from ..options import GcovrConfigOption, Options
+from ..data_model.coverage import FileCoverage
+from ..data_model.container import CoverageContainer
+from ..data_model.merging import get_merge_mode_from_options
+from ..filter import is_file_excluded
+from ..options import GcovrConfigOption, Options, OutputOrDefault
+from ..utils import search_file
 
-from ..coverage import CovData
 
 # the handler
 from .gcov import GcovHandler
@@ -34,13 +38,15 @@ from .html import HtmlHandler
 from .jacoco import JaCoCoHandler
 from .json import JsonHandler
 from .lcov import LcovHandler
+from .markdown import MarkdownHandler
 from .sonarqube import SonarqubeHandler
 from .txt import TxtHandler
 
 LOGGER = logging.getLogger("gcovr")
 
 
-def get_options() -> List[GcovrConfigOption]:
+def get_options() -> list[GcovrConfigOption]:
+    """Get the list of all options from the format handlers."""
     return [
         o
         for o in [
@@ -53,14 +59,16 @@ def get_options() -> List[GcovrConfigOption]:
             *JaCoCoHandler.get_options(),
             *JsonHandler.get_options(),
             *LcovHandler.get_options(),
+            *MarkdownHandler.get_options(),
             *SonarqubeHandler.get_options(),
             *TxtHandler.get_options(),
         ]
-        if not isinstance(o, str)
+        if isinstance(o, GcovrConfigOption)
     ]
 
 
-def validate_options(options) -> CovData:
+def validate_options(options: Options) -> None:
+    """Validate the command line options of the format handlers."""
     for handler in [
         GcovHandler,
         CloverHandler,
@@ -71,31 +79,62 @@ def validate_options(options) -> CovData:
         JaCoCoHandler,
         JsonHandler,
         LcovHandler,
+        MarkdownHandler,
         SonarqubeHandler,
         TxtHandler,
     ]:
         handler(options).validate_options()
 
 
-def read_reports(options) -> CovData:
-    if options.json_add_tracefile or options.cobertura_add_tracefile:
-        covdata: CovData = JsonHandler(options).read_report() or {}
-        covdata.update(CoberturaHandler(options).read_report() or {})
+def read_reports(options: Options) -> CoverageContainer:
+    """Read the reports from the given locations."""
+    if options.json_tracefile or options.cobertura_tracefile:
+        covdata = JsonHandler(options).read_report()
+        covdata.merge(
+            CoberturaHandler(options).read_report(),
+            get_merge_mode_from_options(options),
+        )
     else:
         covdata = GcovHandler(options).read_report()
 
+    if options.include_search_filter:
+        for search_path in options.search_paths or [options.root]:
+            LOGGER.debug(f"Search for included files in {search_path}")
+            for fname in search_file(
+                lambda fname: any(
+                    f.match(fname) for f in options.include_search_filter
+                ),
+                search_path,
+                gcov_exclude_directory=options.gcov_exclude_directory,
+            ):
+                # Return if the filename does not match the filter
+                # Return if the filename matches the exclude pattern
+                if is_file_excluded(
+                    fname, options.include_filter, options.exclude_filter
+                ):
+                    continue
+
+                filecov = FileCoverage("option --include", filename=fname)
+                LOGGER.debug(f"Merge empty coverage data for {fname}")
+                covdata.insert_file_coverage(
+                    filecov, get_merge_mode_from_options(options)
+                )
+
+    if options.merge_lines:
+        covdata.merge_lines()
+
     return covdata
 
 
-def write_reports(covdata: CovData, options: Options):
-    from ..configuration import OutputOrDefault
-
-    Generator = Tuple[
-        List[Optional[OutputOrDefault]],
-        Callable[[CovData, str], bool],
-        Callable[[], None],
-    ]
-    generators: List[Generator] = []
+def write_reports(covdata: CoverageContainer, options: Options) -> None:
+    """Write the reports to the given locations."""
+    generators: list[
+        tuple[
+            list[Optional[OutputOrDefault]],
+            Callable[[CoverageContainer, str], None],
+            Callable[[], None],
+        ]
+    ] = []
 
     if options.clover or options.clover_pretty:
         generators.append(
@@ -205,7 +244,31 @@ def write_reports(covdata: CovData, options: Options):
             )
         )
 
-    if options.sonarqube:
+    if options.markdown:
+        generators.append(
+            (
+                [options.markdown],
+                MarkdownHandler(options).write_report,
+                lambda: LOGGER.warning(
+                    "Markdown output skipped - "
+                    "consider providing an output file with `--markdown=OUTPUT`."
+                ),
+            )
+        )
+
+    if options.markdown_summary:
+        generators.append(
+            (
+                [options.markdown_summary],
+                MarkdownHandler(options).write_summary_report,
+                lambda: LOGGER.warning(
+                    "Markdown summary output skipped - "
+                    "consider providing an output file with `--markdown-summary=OUTPUT`."
+                ),
+            )
+        )
+
+    if options.sonarqube or options.sonarqube_pretty:
         generators.append(
             (
                 [options.sonarqube],
